/* *******************************************************************
   DMX related functions
   ***************************************************************** */

void updateDmx()
// updates DMX values according to their channel[x] value
// respects masterfader and blackout

{
  //dmx.write(1, random(0,25)*10);                            // Short Demo Program
  //dmx.write(random(1,maxChannels), random(0,25)*10);        // Short Demo Program
  if (isBlackout)
  {
    for (uint16_t j = 1; j<=maxChannels; j++)
    {
      dmx.write(j, 0);                                        // dimms DMX channel to zero - without destroying the channel value!
    }
  }
  else
  {
    for (uint16_t j = 1; j<=maxChannels; j++)                        
    {
      if (EEPROM.read(addrChannels+j*11) & 1)                  // isSlider (MISSING: is EEPROM.read fast enough or should it be kept in RAM???)
      {
        dmx.write(j, channel[j]*master/100);                   // Master has only effect on Slider Channels
      }
      else
      {
        dmx.write(j, channel[j]);                              // 
      }
    }
  }
  dmx.update();
}


void fadeDmx()
//Fade: 
// if the value is even, we go up
// if the value is odd, we go down
{
  //Serial.println(F("D035 Pattern Fade"));
  const byte myStep=4;              //only use even values 2,4,8 ... 
  for (uint16_t j = 1; j<=maxChannels; j++)                        
  {
      if (channel[j]%2==0)          //0 up; 1 down
      {
        if (channel[j]<255-myStep)
        {
          channel[j]=channel[j]+myStep;  
        }
        else
        {
          if (channel[j]>=255-myStep) {channel[j]=255;}     // Higher End: Switch over to odd values
          channel[j]=channel[j]-myStep;                     // so theoretical maximum is 255-4=251
        }
      }
      else
      {
        if (channel[j]>myStep)
        {
          channel[j]=channel[j]-myStep;  
        }
        else
        {
          if (channel[j]<=myStep){channel[j]=0;}           // Lower End: Switch over to even values
          channel[j]=channel[j]+myStep;                    // so theoretical minimum is 0+4=4
        }
      }
      //Serial.print (F("D065 ")); Serial.print (j); Serial.print (F(": ")); Serial.println(channel[j]);
  }
  updateDmx();
}

void insertByte(byte total, unsigned int startAddr, unsigned int maxAddr)
// insert total bytes from startAddr 
// shift bytes to right and don't exceed the maxAddr
{
   for (unsigned int actual = maxAddr; actual>=startAddr+total; actual--)
   {
      EEPROM.write(actual,EEPROM.read(actual-total));
   }
   EEPROM.commit();
   //die eingefügten Stellen bleiben ungelöscht, weil sie eigentlich eh in kürze mit echten Daten beschrieben werden müssten. 
}

void deleteByte(byte total, unsigned int startAddr, unsigned int maxAddr)
// delete total bytes from startAddr 
// shift bytes up to maxAddr to left
{
   for (unsigned int actual = startAddr; actual<=maxAddr-total; actual++)
   {
      EEPROM.write(actual,EEPROM.read(actual+total));
   }
   //delete the rightmost total bytes
   for (unsigned int actual = maxAddr-total; actual<=maxAddr; actual++)
   {
      EEPROM.write(actual,0);
   }
   EEPROM.commit();
}

int getSceneAddr(byte wanted)
{
  //int result=-1;
  unsigned int actual=addrScene;
  if (wanted>0)
  {
    for (unsigned int i = 1;  i<=wanted; i++)
    {
      actual=actual+1+8+EEPROM.read(actual)*2;         //1length + 8memo + length*2
      if (actual<addrSequence && i>sceneCache) {sceneCache=i;}                       // cache the "last" scene number
    }
  }
  //Serial.print(F("D086 getSceneAddr("));Serial.print(wanted);Serial.print(F(") "));Serial.println(actual);
  //MISSING: absichern gegen unter addrScene und über addrSequence-255
  return actual;
}


void saveScene(byte actual)
// save actual channel setting as Scene and save to EEPROM
// save only values <>0 to spare EEPROM memory
{
  
  byte newLength = 0;
  unsigned int myPosition = 0;
  //Länge ermitteln
  for (unsigned int j = 1; j<=maxChannels; j++)                        
  {
    if (channel[j]>0)
    {
      newLength++;
    }
  }
  //my Position ermitteln
  myPosition = getSceneAddr(actual);
  
  //evtl Platz anpassen
  byte oldLength=EEPROM.read(myPosition);
  if (oldLength<newLength)
  { 
    insertByte((newLength-oldLength)*2, myPosition+1+8, addrSequence-1);
  }
  else if (oldLength>newLength)
  {
    deleteByte((oldLength-newLength)*2, myPosition+1+8, addrSequence-1);
  }
  
  //speichern
  EEPROM.write(myPosition,newLength);
 
  unsigned int i = 0;  
  for (unsigned int j = 1; j<=maxChannels; j++)                        
  {
    if (channel[j]>0)
    {
      EEPROM.write((myPosition+1+8)+i*2,j);
      EEPROM.write((myPosition+1+8)+i*2+1,channel[j]);
      i++;
    }
  }
  EEPROM.commit();
  Serial.print(F("D110 newLength "));Serial.print(newLength);Serial.print(F("| oldLength "));Serial.println(oldLength);
}

void loadScene(byte actual)
// Load scene from EEPROM and display
{
  unsigned int myPosition = 0;
  
  //my Position ermitteln
  //myPosition = addrScene;
  myPosition = getSceneAddr(actual);

  //Länge lesen
  byte myLength=EEPROM.read(myPosition);

  memset(channel,0,sizeof(channel)); 
  unsigned int i=0;
  for (i = 0; i<myLength; i++)                        //no local i because want to have a debug print afterwards
  {
      channel[EEPROM.read((myPosition+1+8)+i*2)]=EEPROM.read((myPosition+1+8)+i*2+1);
  }
  Serial.print(F("D184 myLength "));Serial.print(myLength);Serial.print(F("| stored pairs "));Serial.println(i);
  updateDmx();
}

/*
// Unused functions 

void saveSceneRam(byte actual)
// Aktuelle Scene übernehmen und "Speichern"
// Im Prinzip ist es nur das channel array in eine Scene zu übernehmen
// beim Speichern ins EEPROM werfen wir die 0er Werte weg
// MISSING: aktuell wird das über ein separates array simuliert ...
{
  unsigned int i=0;
  //zunächst alles Löschen
  memset(sceneChannel,0,sizeof(sceneChannel));
  memset(sceneValue,0,sizeof(sceneValue));
  for (unsigned int j = 1; j<=maxChannels; j++)                        
  {
    if (channel[j]>0)
    {
      sceneChannel[i]=j;
      sceneValue[i]=channel[j];
      i++;
    }
  }
}


void loadSceneRam(byte actual)
// Load scene from RAM and display
// MISSING - just for testpurposes ;-)
{
  memset(channel,0,sizeof(channel));
  for (unsigned int j = 0; j<=sizeof(sceneChannel); j++)                        
  {
    if (sceneChannel[j]>0)
    {
      channel[sceneChannel[j]]=sceneValue[j];
    }
  }
  updateDmx();
}

*/
